%% Persistence Length Quenched Disorder Model and Contour Length Model Parameter Fitting 

% This MatLab code can be used to fit both persistence and contour length
% data of complexes formed between dsDNA and small ligands, allowing the
% determination of the physicochemical (binding) parameters of the
% interaction from the mechanical data. The calculations are based on the
% methods described in Chapter 9 of the book "DNA interactions with drugs
% and other small ligands: single molecule approaches and techniques" by
% Mrcio S. Rocha (Elsevier, 2023). The persistence length quenched
% disorder model is discussed in Section 9.2.3, while the contour length
% method is discussed in Section 9.2.2. Questions can be made directly to
% the author: marcios.rocha@ufv.br

%%%%%%%% Starting the code and input parameters %%%%%%%%%%

clear all % Clear all previous data

cd('D:\Tests\') % Sets your current folder where the code files must be


Data=1; % Select a dataset to perform the fitting. There are two cases: the first one should be used to fit data
%for the persistence length as a function of the ligand concentration, using the function PLQDM (the binding isotherm
%is written and can be changed in this file; the default is the Hill isotherm). The second case is to fit data for the
%contour length as a function of the ligand concentration using the function CL (the binding isotherm
%is written and can be changed in this file; the default is the McGhee - von Hippel isotherm). 

switch Data
    case 1
       % EXAMPLE DATA: Ligand "XYZ" - Persistence length data
        Ct=[0 10 20 30 40 50 60 70 80 90 100]; % Ligand concentrations used in the assay
        y=[1/45 1/44 1/38 1/32 1/31 1/27 1/22 1/21 1/20 1/20 1/20.1]; % Inverse of the corresponding persistence length for each ligand concentration stated above
        Rmax_Cbp=1.2; % Maximum bound ligand fraction multiplied by the DNA base-pair concentration (in units of micromolar) used in the sample (here chosen as a fixed parameter in the fitting; can be made adjustable if needed; can be set to zero if the DNA concentration is very low)
        K=0.015; % Equilibrium association binding constant in units of 1/micromolar (adjustable parameter, set the initial guessed value)
        n=3.6; % Hill exponent (adjustable parameter, set the initial guessed value)
        Xo=0; % Initial point for finding the roots of the binding isotherm (there is no need to change)
        A0=45; % Bare DNA persistence length measured (here chosen as a fixed parameter in the fitting; can be made adjustable if needed)
        A1=17; % Second local persistence length (adjustable parameter, set the initial guessed value)
        beta0=[K n A1]; % Initial values of the guessed parameters used for the iterations 
        lb=[0 0 0];    % Lower bound for the adjustable parameters
        ub=[10 10 30]; % Upper bound for the adjustable parameters

    case 2
        %EXAMPLE DATA: Ligand "ABC" - Contour length data 
        Ct=[0 0.1 0.2 0.25 0.3 0.4 0.5 1 2 3 4 5]; % Ligand concentrations used in the assay
        y=[0 0.055 0.055 0.212 0.236 0.224 0.35 0.45 0.44 0.43 0.45 0.442]; % Relative increase of the measured contour length "Theta = (L - Lo)/Lo" for each above stated ligand concentration
        Cbp=2.4; % DNA base-pair concentration in the sample in units of micromolar
        gamma_Rmax=0.2; % Parameter "gamma" (see definition in Section 9.2.2 of the book) multiplied by the maximum bound ligand fraction (adjustable parameter, set the guessed value)
        K=1; % Equilibrium association binding constant in units of 1/micromolar (adjustable parameter, set the initial guessed value)
        N=1; % Exclusion parameter (binding site size) (adjustable parameter, set the initial guessed value)
        Xo=0;  % Initial point for finding the roots of the binding isotherm (there is no need to change)
        beta0=[gamma_Rmax N K]; % Initial values of the guessed parameters used for the iterations 
        lb=[0 0 0];     % Lower bound for the adjustable parameters
        ub=[10 10 100];  % Upper bound for the adjustable parameters
end

%%%%%%%%%%%% Fitting the data %%%%%%%%%%%%

% The below code call the corresponding functions for persistence OR contour
% length fitting

if Data~=2
    
    % Function for persistence length fittings
    Function=@(Cts,Var) PLQDM(Var,Rmax_Cbp,Cts(1),Cts(2),Xo,A0,Cts(3))
    %Cts are the adjustable parameters; the others are fixed in the fitting
    %(this can be changed)
    my_opts = optimset('Display','final','MaxIter',1000,'Algorithm', 'trust-region-reflective','TolFun', 1e-20,'TolX', 1e-20); % Fitting parameters
    [Fit,resnorm,r,exitflag,output,lambda,J]=lsqcurvefit(Function,beta0,Ct',y',[],[],my_opts); % Call the lsqcurvefit to perform least squares fitting
    %ci = nlparci(Fit,r,'jacobian',J); (optional: can be used to determine the fitting error)
    [Fit'] % Show the best fitting values for the adjustable parameters.
    resnorm   
else
    
    % Function for contour length fittings
    Function=@(Cts,Var) CL(Var,Cbp,Cts(1),Xo,Cts(2),Cts(3))
    %Cts are the adjustable parameters; the others are fixed in the fitting
    %(this can be changed)
    my_opts = optimset('Display','final','MaxIter',1000,'Algorithm', 'trust-region-reflective','TolFun', 1e-20,'TolX', 1e-20); % Fitting parameters
    [Fit,resnorm,r,exitflag,output,lambda,J]=lsqcurvefit(Function,beta0,Ct',y',[],[],my_opts); % Call the lsqcurvefit to perform least squares fitting
    %ci = nlparci(Fit,r,'jacobian',J); (optional: can be used to determine the fitting error)
    [Fit']
    resnorm
end

%%%%%%%%%%   Plotting the data and fitting curve %%%%%%%%%%  

CtCont=linspace(Ct(1),Ct(end),1000); % Define the points of the fitted curve 
FitDat=Function(Fit,CtCont)';
figure()
plot(Ct,y,'ob',CtCont,FitDat,'-r')

 DataMat=[Ct' y'];
 FitMat=[CtCont' FitDat'];
 
 